package Fry::ReadLine::Default;
our $o;
sub setup {$o = $_[1]}
sub addhistory {}
sub prompt {
	my ($class,$prompt) = @_;
	$o->view("\n".$prompt);
	chomp(my $entry = <STDIN>);
	#$o->addhistory($entry);
	return $entry;
}
1;

__END__	

=head1 NAME

Fry::ReadLine::Default -  Default ReadLine plugin for Fry::Shell.

=head1 AUTHOR

Me. Gabriel that is.  I welcome feedback and bug reports to cldwalker AT chwhat DOT com .  If you
like using perl,linux,vim and databases to make your life easier (not lazier ;) check out my website
at www.chwhat.com.


=head1 COPYRIGHT & LICENSE

Copyright (c) 2004, Gabriel Horner. All rights reserved.

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
